package me.mrletsplay.shareclient.util.listeners;

import java.nio.file.Path;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.jface.text.IDocument;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.texteditor.ITextEditor;

import me.mrletsplay.shareclient.ShareClient;
import me.mrletsplay.shareclient.util.ProjectAndPath;
import me.mrletsplay.shareclient.util.ProjectRelativePath;
import me.mrletsplay.shareclient.util.ShareSession;
import me.mrletsplay.shareclient.util.SharedProject;

public class ShareClientPartListener implements IPartListener2 {

	private Map<ProjectAndPath, ShareClientDocumentListener> listeners = new HashMap<>();

	private ShareClientDocumentListener createListener(ProjectAndPath path, IDocument document) {
		if(listeners.containsKey(path)) return listeners.get(path);
		ShareClientDocumentListener listener = new ShareClientDocumentListener(path, document);
		listeners.put(path, listener);
		return listener;
	}

	public void addDocumentListener(IWorkbenchPartReference partRef) {
		IWorkbenchPart part = partRef.getPart(false);
		if(!(part instanceof ITextEditor)) return;
		ITextEditor textEditor = (ITextEditor) part;
		IEditorInput editorInput = textEditor.getEditorInput();
		if(!(editorInput instanceof FileEditorInput)) return;
		FileEditorInput fileEditorInput = (FileEditorInput) editorInput;
		IDocument document = textEditor.getDocumentProvider().getDocument(editorInput);

		IFile file = fileEditorInput.getFile();
		IProject project = file.getProject();

		Path filePath = project.getLocation().toPath().relativize(file.getLocation().toPath());
		ProjectAndPath relPath = new ProjectAndPath(project, filePath.toString());
		document.addDocumentListener(createListener(relPath, document));
		System.out.println("Opened editor: " + relPath);
	}

	private void removeDocumentListener(IWorkbenchPartReference partRef) {
		IWorkbenchPart part = partRef.getPart(false);
		if(!(part instanceof ITextEditor)) return;
		ITextEditor textEditor = (ITextEditor) part;
		IEditorInput editorInput = textEditor.getEditorInput();
		if(!(editorInput instanceof FileEditorInput)) return;
		FileEditorInput fileEditorInput = (FileEditorInput) editorInput;
		IDocument document = textEditor.getDocumentProvider().getDocument(editorInput);

		IFile file = fileEditorInput.getFile();
		IProject project = file.getProject();

		Path filePath = project.getLocation().toPath().relativize(file.getLocation().toPath());
		ProjectAndPath relPath = new ProjectAndPath(project, filePath.toString());
		ShareClientDocumentListener listener = getListener(relPath);
		if(listener == null) return;

		document.removeDocumentListener(listener);
		listeners.remove(relPath);
		System.out.println("Closed editor: " + relPath);
	}

	@Override
	public void partOpened(IWorkbenchPartReference partRef) {
		addDocumentListener(partRef);
	}

	@Override
	public void partInputChanged(IWorkbenchPartReference partRef) {
		addDocumentListener(partRef);
	}

	@Override
	public void partClosed(IWorkbenchPartReference partRef) {
		removeDocumentListener(partRef);
	}

	public Map<ProjectAndPath, ShareClientDocumentListener> getListeners() {
		return listeners;
	}

	public ShareClientDocumentListener getListener(ProjectAndPath path) {
		return listeners.get(path);
	}

	public ShareClientDocumentListener getListener(ProjectRelativePath path) {
		ShareSession session = ShareClient.getDefault().getActiveSession();
		if(session == null) return null;

		SharedProject project = session.getSharedProject(path.projectName());
		if(project == null) return null;

		return listeners.get(new ProjectAndPath(project.getLocal(), path.relativePath()));
	}

}
